/*-
 * Copyright (c) 1999, Peter Wemm <peter@netplex.com.au>
 * All rights reserved.
 * 
 * Local modifications by David P. Reese, Jr. <daver@gomerbud.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $FreeBSD: CVSROOT-src/freebsd/cvswrap.c,v 1.5 2001/11/15 23:36:06 joe Exp $
 */

/*
 * A cheap hack to put a paper thick fence in front of /home/ncvs
 * and a replacement of the ncvs priviliged group.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <pwd.h>
#include <grp.h>
#include <err.h>

#define REALCVS "/usr/bin/cvs"

char linebuf[1024];

int
main(int ac, char **av)
{
	FILE *fp;
	uid_t uid;
	struct passwd *pw;
	struct group *g;
	char *p, *s;
	int karma;

	karma = 0;

	/*
	 * Check that we are setuid root.  We need root privs to mess with
	 * our group id.
	 */
	if (geteuid() != 0)
		err(1, "The cvswrap program must be setuid root.");

	umask(002);

	fp = fopen(ACCESS, "r");
	if (fp == NULL)
		err(1, "cannot open %s", ACCESS);

	uid = getuid();
	pw = getpwuid(uid);
	if (pw == NULL)
		err(1, "you don't exist! (getpwuid(3) failed)");

	while ((p = fgets(linebuf, sizeof(linebuf) - 1, fp)) != NULL) {
		while ((s = strsep(&p, " \t\n")) != NULL) {
			if (*s == '\0')
				continue;	/* whitespace */
			if (*s == '#' || *s == '/' || *s == ';')
				break;		/* comment */
			if (strcmp(s, pw->pw_name) == 0) {
				karma++;
				break;
			}
			break;	/* ignore further tokens on line for now */
		}
	}
	fclose(fp);

	if (karma > 0) {
		/*
		 * Change our gid to that of the cvs repo.  We can then drop
		 * root privs.
		 */
		if ((g = getgrnam(CVSGROUP)) == NULL)
			err(1, "getgrnam(%s)", CVSGROUP);
		if (setgid(g->gr_gid) < 0)
			err(1, "setgid(g->gr_gid) failed!");
		if (seteuid(uid) < 0)
			err(1, "seteuid(uid) failed!");
	} else {
		/*
		 * I'm not certain which OS your cvs server is running, so
		 * it may not support read-only repos.  Bail instead.
		 *
		 * Otherwise, setting the repo to read-only might be useful
		 * for something like a build machine.  This can also be
		 * achieved by setting unavail rules in the avail file, but
		 * it assumes that cvs_acls.pl doesn't have any bugs.
		 */
		errx(1, "You are not in the access file.  CVS access denied.");

		/* NOTREACHED */
		if (setgid(getgid()) < 0)
			err(1, "setgid(getgid()) failed!");
		/* tell cvs to not bother trying to create locks */
		putenv("CVSREADONLYFS=true");
	}

	execv(REALCVS, av);
	err(1, "cannot execv(%s)", REALCVS);
}
